# -*- coding: utf-8 -*-

SAMPLES = [
    {'url': 'http://ppomppu.co.kr/',
     'encoding': 'euc-kr',
     'contains': '장터 리스트'},

    {'url': 'http://www.interpark.com/malls/index.html',
     'encoding': 'euc-kr',
     'contains': '유아동'},

    {'url': 'http://chosun.com/',
     'encoding': 'euc-kr',
     'contains': '사내칼럼'},

    {'url': 'https://www.momoshop.com.tw/main/Main.jsp',
     'encoding': 'MS950',
     'contains': '鞋展獨降'},

    {'url': 'http://www.rakuten.co.jp/',
     'encoding': 'EUC-JP',
     'contains': '送料無料の商品を探す'},

    {'url': 'http://www.qq.com/',
     'encoding': 'GB2312',
     'contains': '今日话题'},

    {'url': 'http://www.kooora.com/',
     'encoding': 'windows-1256',
     'contains': 'أبطال أوروبا'},

    {'url': 'http://www.alwasatnews.com/',
     'encoding': None,
     'contains': 'أوقات الصلاة'},

    {'url': 'https://pantip.com/',
     'encoding': None,
     'contains': 'หน้าแรก'},

    {'url': 'http://manager.co.th/home/',
     'encoding': None,
     'contains': 'ขายสบู่ถูจิ๊มิ๊'},

    {'url': 'http://www.telewebion.com/#!/',
     'encoding': None,
     'contains': 'خروج از سیستم'},

    {'url': 'http://www.varzesh3.com/',
     'encoding': None,
     'contains': 'اخبار فوتبال و والیبال'}
]




from html_to_etree import (_decode_bytes, parse_html_bytes,
                           clean_html, _detect_encoding)

import logging

import pickle

from lxml import etree
from html_text import extract_text

TEST_DATA = None

BLACKLIST = [
    'http://ppomppu.co.kr/',  # cchardet detects 'utf-8' instead of 'euc-kr'
]


def get_data():
    if TEST_DATA:
        return TEST_DATA

    with open('tests/test_data.pkl', 'rb') as fhandle:
        data = pickle.load(fhandle)
        return [x for x in data
                if len(clean_html(x['webdata']['byte_body'])) > 1000
                and x['meta']['url'] not in BLACKLIST]


def test_contains():
    for rec in get_data():
        meta = rec['meta']
        url = meta['url']
        webdata = rec['webdata']

        encoding, uni = _decode_bytes(body=webdata['byte_body'],
                                      content_type=webdata['content-type'])
        contains = meta['contains']
        logging.warning('%s %s', url, encoding)
        assert contains in uni, (url, contains, uni)


def test_tree():
    for rec in get_data():
        meta = rec['meta']
        url = meta['url']
        contains = meta['contains']

        webdata = rec['webdata']
        body = webdata['byte_body']
        content_type = webdata['content-type']

        if len(clean_html(body)) < 100:
            logging.warning('skipping %s', url)
            continue

        tree = parse_html_bytes(body=body,
                                content_type=content_type)

        assert contains in extract_text(tree), (
            url, contains, etree.tostring(tree, encoding='utf-8'))



# -*- coding: utf-8 -*-

"""
parse a html document into an lxml etree

Notes:
- https://github.com/buriy/python-readability/issues/42
- https://github.com/scrapy/scrapy/blob/c68140e68a7aee894e304122e2532c2fc7edfdc5/scrapy/http/response/text.py#L69
"""

__author__ = 'Johannes Ahlmann'
__email__ = 'johannes@fluquid.com'

import lxml
import lxml.etree
from lxml.html.clean import Cleaner
from w3lib.encoding import html_to_unicode, resolve_encoding
import cchardet as chardet


# 'cleaner' settings used by teamhg-memex/html-text
clean_html = Cleaner(
    scripts=True,
    javascript=False,  # onclick attributes are fine
    comments=True,
    style=True,
    links=True,
    meta=True,
    page_structure=False,  # <title> may be nice to have
    processing_instructions=True,
    embedded=True,
    frames=True,
    forms=False,  # keep forms
    annoying_tags=False,
    remove_unknown_tags=False,
    safe_attrs_only=False,
).clean_html


def _detect_encoding(bytestring, default_encoding='utf-8'):
    # NOTE: alternatively `UnicodeDammit(x).originalEncoding`
    # NOTE: alternatively use scrapy.http.TextResponse().text
    encoding = chardet.detect(bytestring).get('encoding')
    if encoding:
        # TODO: `resolve_encoding`?
        return resolve_encoding(encoding)
    else:
        return default_encoding


def _decode_bytes(body, content_type='', default_encoding='utf-8'):
    encoding, uni_string = html_to_unicode(
        content_type_header=content_type,
        html_body_str=body,
        default_encoding=default_encoding,
        auto_detect_fun=_detect_encoding)
    return (encoding, uni_string)


def parse_html_bytes(body, content_type='', default_encoding='utf-8'):
    encoding, uni_string = _decode_bytes(body, content_type, default_encoding)
    return parse_html_unicode(uni_string)


def parse_html_unicode(uni_string):
    parser = lxml.html.HTMLParser(encoding='utf8')
    return lxml.html.fromstring(uni_string.encode('utf8'), parser=parser)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import pickle

import requests
from tqdm import tqdm

import metadata

HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36'
}

def fetch_data(url):
    try:
        res = requests.get(url,
                           headers=HEADERS,
                           timeout=10)
        content_type = res.headers.get('content-type')
        raw = res.content
        return {'content-type': content_type,
                'byte_body': raw}
    except Exception as e:
        logging.exception(e)
        return None

def main():
    data = [{'meta': rec,
             'webdata': fetch_data(rec['url'])}
            for rec in tqdm(metadata.SAMPLES)
            if rec
           ]

    with open('test_data.pkl', 'wb') as fhandle:
        pickle.dump(data, fhandle, protocol=2)

main()
